@@ -45,7 +45,8 @@ pub trait TokenReader<T: PartialEq, TData> {
 /// Trait for a sender that can append a token to a sequence
 pub trait TokenSender<T: PartialEq, TData> {
     /// Appends a new [`Token`]
-    fn push(&mut self, token: Token<T, TData>);
+    /// Will return false if could not push token
+    fn push(&mut self, token: Token<T, TData>) -> bool;
 }
 
 mod buffered_token_queue {
@@ -65,8 +66,9 @@ mod buffered_token_queue {
     }
 
     impl<T: PartialEq, TData> TokenSender<T, TData> for BufferedTokenQueue<T, TData> {
-        fn push(&mut self, token: Token<T, TData>) {
-            self.buffer.push_back(token)
+        fn push(&mut self, token: Token<T, TData>) -> bool {
+            self.buffer.push_back(token);
+            true
         }
     }
 
@@ -96,7 +98,7 @@ mod parallel_token_queue {
     use super::*;
     use std::sync::mpsc::{sync_channel, Receiver, RecvError, SyncSender};
 
-    const STREAMED_CHANNEL_BUFFER_SIZE: usize = 20;
+    const DEFAULT_BUFFER_SIZE: usize = 20;
 
     /// A token queue used for doing lexing and parsing on different threads. Will send tokens between threads
     pub struct ParallelTokenQueue;
@@ -106,7 +108,13 @@ mod parallel_token_queue {
         /// sender is on the lexer thread
         pub fn new<T: PartialEq, TData>(
         ) -> (ParallelTokenSender<T, TData>, ParallelTokenReader<T, TData>) {
-            let (sender, receiver) = sync_channel::<Token<T, TData>>(STREAMED_CHANNEL_BUFFER_SIZE);
+            Self::new_with_buffer_size(DEFAULT_BUFFER_SIZE)
+        }
+
+        pub fn new_with_buffer_size<T: PartialEq, TData>(
+            buffer_size: usize,
+        ) -> (ParallelTokenSender<T, TData>, ParallelTokenReader<T, TData>) {
+            let (sender, receiver) = sync_channel::<Token<T, TData>>(buffer_size);
             (
                 ParallelTokenSender(sender),
                 ParallelTokenReader {
@@ -129,8 +137,8 @@ mod parallel_token_queue {
     }
 
     impl<T: PartialEq, TData> TokenSender<T, TData> for ParallelTokenSender<T, TData> {
-        fn push(&mut self, token: Token<T, TData>) {
-            self.0.send(token).unwrap();
+        fn push(&mut self, token: Token<T, TData>) -> bool {
+            self.0.send(token).is_ok()
         }
     }
 
@@ -158,9 +166,9 @@ mod parallel_token_queue {
         fn scan(&mut self, mut cb: impl FnMut(&T, &TData) -> bool) -> Option<&Token<T, TData>> {
             let found = scan_cache(&mut self.cache, &mut cb);
             let mut return_next = match found {
-                ScanCacheResult::FoundInCache(i) => return self.cache.get(i),
-                ScanCacheResult::NotFound => false,
+                ScanCacheResult::RetrievableInCacheAt(idx) => return self.cache.get(idx),
                 ScanCacheResult::Found => true,
+                ScanCacheResult::NotFound => false,
             };
             loop {
                 match self.receiver.recv() {
@@ -216,8 +224,9 @@ mod generator_token_queue {
     }
 
     impl<'a, T: PartialEq, TData> TokenSender<T, TData> for GeneratorTokenQueueBuffer<'a, T, TData> {
-        fn push(&mut self, token: Token<T, TData>) {
+        fn push(&mut self, token: Token<T, TData>) -> bool {
             self.0.push_back(token);
+            true
         }
     }
 
@@ -271,9 +280,9 @@ mod generator_token_queue {
             let cb = &mut cb;
             let found = scan_cache(&mut self.cache, cb);
             let mut return_next = match found {
-                ScanCacheResult::FoundInCache(i) => return self.cache.get(i),
-                ScanCacheResult::NotFound => false,
+                ScanCacheResult::RetrievableInCacheAt(idx) => return self.cache.get(idx),
                 ScanCacheResult::Found => true,
+                ScanCacheResult::NotFound => false,
             };
             let mut found = None::<usize>;
             while found.is_none() {
@@ -300,32 +309,33 @@ mod generator_token_queue {
     }
 }
 
-enum ScanCacheResult<T> {
+enum ScanCacheResult {
+    RetrievableInCacheAt(usize),
     NotFound,
-    FoundInCache(T),
     // Aka pull out next one
     Found,
 }
 
+/// Returns the idx of the **next item** after cb returns true
+/// This returns the idx instead of the item for lifetime reasons
 fn scan_cache<T: PartialEq, TData>(
     cache: &mut VecDeque<Token<T, TData>>,
     cb: &mut impl FnMut(&T, &TData) -> bool,
-) -> ScanCacheResult<usize> {
-    // Scan cache first
-    let mut found = None;
-    for (i, token) in cache.iter().enumerate() {
+) -> ScanCacheResult {
+    let mut cb_returned_true_at_idx = None::<usize>;
+    // Try to find in idx. Returns the idx when found
+    for (idx, token) in cache.iter().enumerate() {
         if cb(&token.0, &token.1) {
-            found = Some(i);
+            cb_returned_true_at_idx = Some(idx);
             break;
         }
     }
-    if let Some(i) = found {
-        if i < cache.len() {
-            return ScanCacheResult::FoundInCache(i);
+    if let Some(idx) = cb_returned_true_at_idx {
+        if idx + 1 < cache.len() {
+            ScanCacheResult::RetrievableInCacheAt(idx + 1)
+        } else {
+            ScanCacheResult::Found
         }
-    }
-    if found.is_some() {
-        ScanCacheResult::Found
     } else {
         ScanCacheResult::NotFound
     }
@@ -500,7 +510,9 @@ mod tests {
         fn lexer(state: &mut u8, sender: &mut GeneratorTokenQueueBuffer<u8, ()>) {
             *state += 1;
             match state {
-                1 | 2 | 3 => sender.push(Token(*state * 2, ())),
+                1 | 2 | 3 => {
+                    sender.push(Token(*state * 2, ()));
+                }
                 _ => {}
             }
         }
